@@ -11,12 +11,14 @@ from __future__ import absolute_import
 import logging
 
 from django.conf import settings
+from django.core.exceptions import SuspiciousOperation
 from django.http import HttpRequest
 from django.template import TemplateSyntaxError
 from django.template.loader import LoaderOrigin
 
 from raven.base import Client
-from raven.contrib.django.utils import get_data_from_template
+from raven.contrib.django.utils import get_data_from_template, get_host
+from raven.contrib.django.middleware import SentryLogMiddleware
 from raven.utils.wsgi import get_headers, get_environ
 
 __all__ = ('DjangoClient',)
@@ -28,65 +30,105 @@ class DjangoClient(Client):
     def is_enabled(self):
         return bool(self.servers or 'sentry' in settings.INSTALLED_APPS)
 
-    def get_user_info(self, request):
-        if request.user.is_authenticated():
-            user_info = {
-                'is_authenticated': True,
-                'id': request.user.pk,
-                'username': request.user.username,
-                'email': request.user.email,
-            }
-        else:
-            user_info = {
-                'is_authenticated': False,
-            }
+    def get_user_info(self, user):
+        if not user.is_authenticated():
+            return {'is_authenticated': False}
+
+        user_info = {
+            'id': user.pk,
+            'is_authenticated': True,
+        }
+
+        if hasattr(user, 'email'):
+            user_info['email'] = user.email
+
+        if hasattr(user, 'get_username'):
+            user_info['username'] = user.get_username()
+        elif hasattr(user, 'username'):
+            user_info['username'] = user.username
+
         return user_info
 
     def get_data_from_request(self, request):
-        from django.contrib.auth.models import User, AnonymousUser
+        try:
+            from django.contrib.auth.models import AbstractBaseUser as BaseUser
+        except ImportError:
+            from django.contrib.auth.models import User as BaseUser  # NOQA
+
+        result = {}
+
+        if hasattr(request, 'user') and isinstance(request.user, BaseUser):
+            result['sentry.interfaces.User'] = self.get_user_info(request.user)
+
+        try:
+            uri = request.build_absolute_uri()
+        except SuspiciousOperation:
+            # attempt to build a URL for reporting as Django won't allow us to
+            # use get_host()
+            if request.is_secure():
+                scheme = 'https'
+            else:
+                scheme = 'http'
+            host = get_host(request)
+            uri = '%s://%s%s' % (scheme, host, request.path)
 
         if request.method != 'GET':
             try:
-                data = request.raw_post_data and request.raw_post_data or request.POST
-            except Exception:
-                # assume we had a partial read:
-                data = '<unavailable>'
+                data = request.body
+            except:
+                try:
+                    data = request.raw_post_data
+                except Exception:
+                    # assume we had a partial read.
+                    try:
+                        data = request.POST or '<unavailable>'
+                    except Exception:
+                        data = '<unavailable>'
         else:
             data = None
 
         environ = request.META
 
-        result = {
+        result.update({
             'sentry.interfaces.Http': {
                 'method': request.method,
-                'url': request.build_absolute_uri(),
+                'url': uri,
                 'query_string': request.META.get('QUERY_STRING'),
                 'data': data,
                 'cookies': dict(request.COOKIES),
                 'headers': dict(get_headers(environ)),
                 'env': dict(get_environ(environ)),
             }
-        }
-
-        if hasattr(request, 'user') and isinstance(request.user, (User, AnonymousUser)):
-            result['sentry.interfaces.User'] = self.get_user_info(request)
+        })
 
         return result
 
     def build_msg(self, *args, **kwargs):
         data = super(DjangoClient, self).build_msg(*args, **kwargs)
 
-        stacktrace = data.get('sentry.interfaces.Stacktrace')
-        if stacktrace:
+        stacks = (
+            data.get('sentry.interfaces.Stacktrace'),
+            data.get('sentry.interfaces.Exception', {}).get('stacktrace'),
+        )
+
+        for stacktrace in filter(bool, stacks):
             for frame in stacktrace['frames']:
-                if frame.get('module', '').startswith('django.'):
+                module = frame.get('module')
+                if not module:
+                    continue
+
+                if module.startswith('django.'):
                     frame['in_app'] = False
 
-        if 'django.contrib.sites' in settings.INSTALLED_APPS:
-            from django.contrib.sites import models as site_models
-            site = site_models.Site.objects.get_current()
-            site_name = site.name or site.domain
-            data['tags'].setdefault('site', site_name)
+        if not self.site and 'django.contrib.sites' in settings.INSTALLED_APPS:
+            try:
+                from django.contrib.sites.models import Site
+                site = Site.objects.get_current()
+                site_name = site.name or site.domain
+                data['tags'].setdefault('site', site_name)
+            except Exception:
+                # Database error? Fallback to the id
+                data['tags'].setdefault('site', settings.SITE_ID)
 
         return data
 
@@ -96,6 +138,9 @@ class DjangoClient(Client):
         else:
             data = kwargs['data']
 
+        if request is None:
+            request = getattr(SentryLogMiddleware.thread, 'request', None)
+
         is_http_request = isinstance(request, HttpRequest)
         if is_http_request:
             data.update(self.get_data_from_request(request))
@@ -135,8 +180,10 @@ class DjangoClient(Client):
         elif 'sentry' in settings.INSTALLED_APPS:
             try:
                 return self.send_integrated(kwargs)
-            except Exception, e:
-                self.error_logger.error('Unable to record event: %s', e, exc_info=True)
+            except Exception as e:
+                self.error_logger.error(
+                    'Unable to record event: %s\nEvent was: %r', e,
+                    kwargs['message'], exc_info=True)
 
     def send_integrated(self, kwargs):
         from sentry.models import Group
